@@ -19,7 +19,7 @@
  * Copyright (c) 2015      Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
  * Copyright (c) 2021-2023 Nanook Consulting.  All rights reserved.
- * Copyright (c) 2022      Triad National Security, LLC. All rights reserved.
+ * Copyright (c) 2022-2023 Triad National Security, LLC. All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -46,6 +46,7 @@
 #include "src/mca/plog/base/base.h"
 #include "src/mca/pnet/base/base.h"
 #include "src/mca/preg/base/base.h"
+#include "src/mca/prm/base/base.h"
 #include "src/mca/psec/base/base.h"
 #include "src/mca/psquash/base/base.h"
 #include "src/mca/pstrg/base/base.h"
@@ -53,6 +54,7 @@
 #include "src/util/pmix_name_fns.h"
 #include "src/util/pmix_net.h"
 #include "src/util/pmix_output.h"
+#include "src/util/pmix_printf.h"
 #include "src/util/pmix_show_help.h"
 
 #include "src/client/pmix_client_ops.h"
@@ -114,8 +116,7 @@ PMIX_EXPORT pmix_globals_t pmix_globals = {
     .external_topology = false,
     .external_progress = false,
     .iof_flags = PMIX_IOF_FLAGS_STATIC_INIT,
-    .keyindex = PMIX_POINTER_ARRAY_STATIC_INIT,
-    .next_keyid = PMIX_INDEX_BOUNDARY
+    .keyindex = PMIX_KEYINDEX_STATIC_INIT
 };
 
 static void _notification_eviction_cbfunc(struct pmix_hotel_t *hotel, int room_num, void *occupant)
@@ -128,6 +129,20 @@ static void _notification_eviction_cbfunc(struct pmix_hotel_t *hotel, int room_n
 
 static bool util_initialized = false;
 
+void pmix_expose_param(char *param)
+{
+    char *value, *pm;
+
+    value = strchr(param, '=');
+    *value = '\0';
+    ++value;
+    pmix_asprintf(&pm, "PMIX_MCA_%s", param);
+    setenv(pm, value, true);
+    free(pm);
+    --value;
+    *value = '=';
+}
+
 int pmix_init_util(pmix_info_t info[], size_t ninfo, char *libdir)
 {
     pmix_status_t ret;
@@ -175,6 +190,12 @@ int pmix_init_util(pmix_info_t info[], size_t ninfo, char *libdir)
         return ret;
     }
 
+    /* register params for pmix */
+    if (PMIX_SUCCESS != (ret = pmix_register_params())) {
+        fprintf(stderr, "pmix_register_params failed\n");
+        return ret;
+    }
+
     /* initialize the mca */
     if (PMIX_SUCCESS != (ret = pmix_mca_base_open(libdir))) {
         fprintf(stderr, "pmix_mca_base_open failed\n");
@@ -225,12 +246,6 @@ int pmix_rte_init(uint32_t type, pmix_info_t info[], size_t ninfo, pmix_ptl_cbfu
         return PMIX_ERROR;
     }
 
-    /* register params for pmix */
-    if (PMIX_SUCCESS != (ret = pmix_register_params())) {
-        fprintf(stderr, "pmix_register_params failed\n");
-        return ret;
-    }
-
     /* scan incoming info for directives */
     if (NULL != info) {
         for (n = 0; n < ninfo; n++) {
@@ -313,9 +328,8 @@ int pmix_rte_init(uint32_t type, pmix_info_t info[], size_t ninfo, pmix_ptl_cbfu
     ret = pmix_hotel_init(&pmix_globals.notifications, pmix_globals.max_events, pmix_globals.evbase,
                           pmix_globals.event_eviction_time, _notification_eviction_cbfunc);
     PMIX_CONSTRUCT(&pmix_globals.nspaces, pmix_list_t);
-    PMIX_CONSTRUCT(&pmix_globals.keyindex, pmix_pointer_array_t);
-    pmix_pointer_array_init(&pmix_globals.keyindex, 1024, INT_MAX, 128);
-    /* tools as well as servers can belong to groups */
+    PMIX_CONSTRUCT(&pmix_globals.keyindex, pmix_keyindex_t);
+    pmix_pointer_array_init(pmix_globals.keyindex.table, 1024, INT_MAX, 128);
     PMIX_CONSTRUCT(&pmix_client_globals.groups, pmix_list_t);
     /* need to hold off checking the hotel init return code
      * until after we construct all the globals so they can
@@ -549,6 +563,19 @@ int pmix_rte_init(uint32_t type, pmix_info_t info[], size_t ninfo, pmix_ptl_cbfu
         goto return_error;
     }
 
+    /* open and initialize */
+    ret = pmix_mca_base_framework_open(&pmix_prm_base_framework, PMIX_MCA_BASE_OPEN_DEFAULT);
+    if (PMIX_SUCCESS != ret) {
+        error = "pmix_prm_base_open";
+        goto return_error;
+    }
+
+    ret = pmix_prm_base_select();
+    if (PMIX_SUCCESS != ret) {
+        error = "pmix_prm_base_select";
+        goto return_error;
+    }
+
     /* initialize the attribute support system */
     pmix_init_registered_attrs();
 
